import psutil


class SystemMemory:
    
    def __init__(self, value=None):
        self._value = psutil.virtual_memory() if value is None else value

    @property
    def total(self):
        return ByteValue(self._value.total)
    
    @property
    def available(self):
        return ByteValue(self._value.available, self._value.total)

    @property
    def used(self):
        return ByteValue(self._value.used, self._value.total)

    @property
    def free(self):
        return ByteValue(self._value.free, self._value.total)

    @property
    def active(self):
        return ByteValue(self._value.active, self._value.total)

    @property
    def inactive(self):
        return ByteValue(self._value.inactive, self._value.total)

    @property
    def buffers(self):
        return ByteValue(self._value.buffers, self._value.total)

    @property
    def cached(self):
        return ByteValue(self._value.cached, self._value.total)

    @property
    def shared(self):
        return ByteValue(self._value.shared, self._value.total)

    @property
    def slab(self):
        return ByteValue(self._value.slab, self._value.total)

class ByteValue:
    
    def __init__(self, value, total=None, unit_divisor=1000.0):
        self._value = value
        self._total = value if total is None else total
        self._unit_divisor = unit_divisor

    @property
    def bytes(self):
        return self._value

    @property
    def kilobytes(self):
        return self.bytes / self._unit_divisor

    @property
    def megabytes(self):
        return self.kilobytes / self._unit_divisor

    @property
    def gigabytes(self):
        return self.megabytes / self._unit_divisor

    @property
    def terabytes(self):
        return self.gigabytes / self._unit_divisor

    @property
    def percent(self):
        return self._value / self._total * 100.0