from memory.memory import SystemMemory, ByteValue
from memory.error import NegativeAllocationPercentException


class Service:

    def __init__(self, name, allocation_percent, total_memory):
        self._name = name
        self._allocation_percent = allocation_percent
        self._total_memory = total_memory

    @property
    def name(self):
        return self._name

    @property
    def allocation(self):

        allocation = (self._allocation_percent / 100.0) * self._total_memory.bytes

        return ByteValue(allocation, self._total_memory.bytes)

    @property
    def allocation_percent(self):
        return self._allocation_percent

    @allocation_percent.setter
    def allocation_percent(self, percent):

        if percent < 0.0:
            raise NegativeAllocationPercentException()

        self._allocation_percent = percent

    def to_dict(self):
        return {
            'name': self._name, 
            'percent': self._allocation_percent
        }


class ServiceFinder:

    def __init__(self, config, system_memory=SystemMemory()):
        self._config = config
        self._system_memory = system_memory

    def find_all(self):
        return self._config.memory.allocation.services

    def find_by_name(self, name):

        services = self.find_all()

        found_service = list(filter(lambda u:u.name == name, services))

        if len(found_service) > 0:
            return found_service[0]
        else:
            return None
